(**************************************************************************)
(*                                                                        *)
(*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       *)
(*  Kim Nguyen, Laurent Sartre                                            *)
(*                                                                        *)
(*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      *)
(*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 *)
(*                                                                        *)
(*  https://www.informatique-mpi.fr/                                      *)
(*                                                                        *)
(**************************************************************************)

(* Compression de texte avec l'algorithme Lempel--Ziv--Welch *)

type compressed = string
(* Ici, un texte compressé est une chaîne de caractères '0' et '1'.
   Dans une implémentation réaliste, ce serait des bits. *)

(* Dans cette version,
   - l'alphabet est limité aux caractères '0' et '1' (en pratique, ce serait
     les bits de l'entrée) ;
   - la taille des codes est variable. *)
val encode: string -> compressed
val decode: compressed -> string

(* Dans variante,
   - l'alphabet est l'ASCII ;
   - la taille des codes est fixe et passée comme premier argument (`size`).

   Chaque code est écrit sur exactement `size` bits.
   Le nombre de codes est donc limité à 2^size.
   Une fois cette limite atteinte, on ne génère plus de nouveaux codes. *)
module FixedSize: sig
  val encode: int -> string -> compressed
  val decode: int -> compressed -> string

  val decode_and_search: int -> compressed -> string -> int
  (* `decode_and_search size c m` décompresse `c`, en signalant toute
     occurrence du motif `m` dans le texte décompressé, et renvoie le
     nombre total d'occurrences. Ne stocke que M caractères, où M est la
     taille du motif, i.e. on ne stocke pas l'intégralité du texte
     décompressé. *)
end

